import { EntityEquippableComponent, EntityInventoryComponent, EquipmentSlot, GameMode } from "@minecraft/server";
import { ArmorSlotEnum } from "./armor/exports";
import constants from "./constants";

function hasEquipment(player, equipmentSlot, itemIdentifier) {
    const playerEquipmentComponent = player.getComponent(EntityEquippableComponent.componentId);
    const itemStack = playerEquipmentComponent.getEquipment(equipmentSlot);

    if (!itemStack || itemStack?.typeId != itemIdentifier) return false;
    return true;
}

function findItemsWithId(player, itemIdentifier) {
    const playerInventoryContainer = player.getComponent(EntityInventoryComponent.componentId).container;
    const items = [];

    for (let slot = 0; slot < playerInventoryContainer.size; slot++) {
        const itemStack = playerInventoryContainer.getItem(slot);

        if (!itemStack || itemStack?.typeId != itemIdentifier) continue;
        items.push({ slot, itemStack });
    }
    return items;
}

function getArmorItemIdentifier(itemPrefix, armorSlot) {
    return `${itemPrefix}_${ArmorSlotEnum[armorSlot]}`;
}

function getArmor(player) {
    const playerEquipmentComponent = player.getComponent(EntityEquippableComponent.componentId);
    const itemStacks = [];

    for (const slot of Object.keys(EquipmentSlot).slice(0, -2)) {
        const item = playerEquipmentComponent.getEquipment(slot);

        if (!item) continue;
        itemStacks.push(item);
    }
    return itemStacks;
}

function calculateKnockback(attackerPosition, playerFinalPosition, threshold = 0.1) {
    const dx = playerFinalPosition.x - attackerPosition.x;
    const dy = playerFinalPosition.y - attackerPosition.y;
    const dz = playerFinalPosition.z - attackerPosition.z;
    const distance = Math.sqrt(dx * dx + dy * dy + dz * dz);
    const knockback = distance * threshold; // Scale factor to adjust the knockback
    return knockback;
}

function getArmorTreshold(itemStack) {
    return Object.entries(constants.armorKnockbacks).map(([k, v]) => itemStack.typeId.includes(k) ? v : -1).find(n => n != -1) ?? -1;
}

export function damageItem(player, slot, damage = 1) {
    const playerEquipmentComponent = player.getComponent(EntityEquippableComponent.componentId);
    const itemStack = playerEquipmentComponent.getEquipment(slot);
  
    if (!itemStack?.typeId.startsWith('true_dn:') || player.getGameMode() === GameMode.creative) return;
    const durability = itemStack.getComponent('durability');
    const enchantable = itemStack.getComponent('enchantable');
    const unbreaking = enchantable?.getEnchantment('unbreaking')?.level ?? 0;
  
    if (Math.random() * 100 <= (100 / (unbreaking + 1))) durability.damage += damage;
    if (durability.damage > durability.maxDurability) {
      return playerEquipmentComponent.setEquipment(EquipmentSlot.Mainhand, player.playSound('random.break'));
    }
    playerEquipmentComponent.setEquipment(EquipmentSlot.Mainhand, itemStack);
  }

export {
    hasEquipment,
    getArmorItemIdentifier,
    findItemsWithId,
    calculateKnockback,
    getArmor,
    getArmorTreshold
}