import { system, world, EntityInventoryComponent, EntityEquippableComponent, Player, EquipmentSlot, EntityDamageCause } from "@minecraft/server";
import { Parcanite } from "./armor/exports";
import { findItemsWithId, getArmorItemIdentifier, hasEquipment, getArmor, calculateKnockback, getArmorTreshold, damageItem } from "./functions";
import AntiBurner from "./burn";
import constants from "./constants";
import WoodConvertions  from "./wood_convertions";


system.runInterval(() => {
    for (const player of world.getPlayers()) {
        for (const equipmentSlot of Object.keys(Parcanite).slice(1)) {
            const itemOptions = Parcanite[equipmentSlot];
            const playerInventoryContainer = player.getComponent(EntityInventoryComponent.componentId).container;
            const playerEquipmentComponent = player.getComponent(EntityEquippableComponent.componentId);
            
            // Inventory ---
            findItemsWithId(player, getArmorItemIdentifier(Parcanite.id, equipmentSlot))
            .map(found => {
                found.itemStack.setLore(itemOptions.lore);
                playerInventoryContainer.setItem(found.slot, found.itemStack);
            });
            
            // Equipment ---
            if (!hasEquipment(player, equipmentSlot, getArmorItemIdentifier(Parcanite.id, equipmentSlot))) continue;
            itemOptions?.command ? player.runCommandAsync(itemOptions.command) : null;
            itemOptions?.effect ? player.addEffect(itemOptions.effect.id, itemOptions.effect.duration, itemOptions.effectOptions): null;
            
            if (!itemOptions.lore) continue;
            const armorItemStack = playerEquipmentComponent.getEquipment(equipmentSlot);

            armorItemStack.setLore(itemOptions.lore);
            playerEquipmentComponent.setEquipment(equipmentSlot, armorItemStack);
        }
    }
}, 0);


world.afterEvents.playerBreakBlock.subscribe(({ player }) => damageItem(player, EquipmentSlot.Mainhand));

world.afterEvents.entityHurt.subscribe(event => {
    const { hurtEntity, damageSource: { cause, damagingEntity } } = event;
  
    if (damagingEntity instanceof Player && cause === EntityDamageCause.entityAttack) damageItem(damagingEntity, EquipmentSlot.Mainhand);
    if (!(hurtEntity instanceof Player) || !damagingEntity) return;
    try {
        const { x, z } = hurtEntity.getViewDirection();
        const armorPieces = getArmor(hurtEntity).filter(item => getArmorTreshold(item) != -1);
        const knockbackTresh = armorPieces.map(item => getArmorTreshold(item)).reduce((prev, current) => prev + current);
        
       hurtEntity.applyKnockback(x, z, -calculateKnockback(damagingEntity.location, hurtEntity.location, knockbackTresh), 0)

    } catch {}
})

world.beforeEvents.itemUseOn.subscribe(({ source: player, itemStack: item, block }) => {
    if (!item.typeId.startsWith('true_dn:')) return;
    const tags = block.getTags();
    const blockIdentifier = block.typeId;
  
    system.run(() => {
        switch (true) {
            case constants.tools.hoe.includes(item.typeId): {
                if (!tags.includes('grass') && !blockIdentifier.includes('dirt'))
                    return;
                player.playSound('step.gravel');
                break;
            }
            case constants.tools.axe.includes(item.typeId): { 
                if (!Object.keys(WoodConvertions).includes(blockIdentifier))
                    return;
                player.playSound('use.wood');
                break;
            }
            case constants.tools.shovel.includes(item.typeId): {
                if (!tags.includes('grass') || blockIdentifier.includes("path"))
                    return;
                block.setType("minecraft:grass_path")
                player.playSound('step.grass');
                break;
            }
            default: return;
        }

        damageItem(player, EquipmentSlot.Mainhand);
    });
});

system.run(() => new AntiBurner([ "parcanite", "deathnerite" ]));